function [Q, beta] = BioFor2(New_Atom, Q, beta, k )
% BioFor2 enlargers the dual/biorthogonal basis by one vector
%  
% It modifies the biorthogonal basis and updates the set of orthonormal vectors spanning
% the same space. 
%  
% Usage:     [ Q, beta ] = BioFor(  New_Atom, Q, beta, k );
%  
% Inputs:
%   New_Atom to enlarge the biortho-basis and subspace
%   Q       Q orthonormal basis
%   beta    biorthogonal basis 
%   k       number of elements in the  enlarged basis
%  
% Outputs: 
%   Q       augmented orthonormal basis 
%   beta    augmented biorthogonal basis
%  
% References:
%   L. Rebollo-Neira, "Recursive bi-orthogonalisation approach and orthogonal projectors",
%   math-ph/0209026 (2002).  
%  
%orthogonalization of D(:,k) w.r.t Q(:,1),..., Q(:,k-1)
if k==1 Q(:,k)=New_Atom;end
for p=1:k-1
  Q(:,k)= New_Atom-(Q(:,p)'*New_Atom)*Q(:,p);
end
%re-orthogonalization of D(:,k) w.r.t Q(:,1),..., Q(:,k-1)
for p=1:k-1
  Q(:,k)=Q(:,k)-(Q(:,p)'*Q(:,k))*Q(:,p);
end

nork=norm(Q(:,k)); 
Q(:,k)=Q(:,k)/nork; %normalization of Q(:,k) 

%updating of biorthogonal functions
if k>1
  beta=beta-Q(:,k)*(New_Atom'*beta)/nork; 
end	
beta(:,k)=Q(:,k)/nork;   % k-th biorthogonal function
  
%Last modification L. Rebollo-Neira (2014)
%This program is free software; you can redistribute it and/or modify it under the terms 
%of the GNU General Public License as published by the Free Software Foundation; either 
%version 2 of the License, or (at your option) any later version.
%
%This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
%without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
%See the GNU General Public License for more details.
%
%You should have received a copy of the GNU General Public License along with this program;
%if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
%Boston, MA  02110-1301, USA.
